package ch.frankel.blog.wsinject;

import static javax.xml.datatype.DatatypeConstants.FIELD_UNDEFINED;
import static org.springframework.ws.test.server.RequestCreators.withPayload;
import static org.springframework.ws.test.server.ResponseMatchers.payload;

import java.util.GregorianCalendar;
import java.util.Random;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.transform.Source;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.testng.AbstractTestNGSpringContextTests;
import org.springframework.ws.test.server.MockWebServiceClient;
import org.springframework.xml.transform.StringSource;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

@ContextConfiguration(locations = { "classpath:/spring-ws-servlet.xml", "classpath:/applicationContext.xml" })
public class FindPersonServiceEndpointIT extends AbstractTestNGSpringContextTests {

	@Autowired
	private ApplicationContext applicationContext;

	private MockWebServiceClient client;

	@BeforeMethod
	protected void setUp() {

		client = MockWebServiceClient.createClient(applicationContext);
	}

	@DataProvider
	protected Object[][] data() throws DatatypeConfigurationException {

		int size = 5;

		Object[][] data = new Object[size][2];

		for (int i = 0; i < size; i++) {

			int j = new Random().nextInt(i * 5 + 10);

			String request = "<a:findPersonRequestType xmlns:a='http://blog.frankel.ch/ws-inject'><id>"
					+ j + "</id></a:findPersonRequestType>";

			GregorianCalendar calendar = new GregorianCalendar();

			XMLGregorianCalendar xmlCalendar = DatatypeFactory.newInstance()
					.newXMLGregorianCalendar(calendar);

			xmlCalendar.setHour(FIELD_UNDEFINED);
			xmlCalendar.setMinute(FIELD_UNDEFINED);
			xmlCalendar.setSecond(FIELD_UNDEFINED);
			xmlCalendar.setMillisecond(FIELD_UNDEFINED);

			String expectedResponse = "<ns3:findPersonResponseType xmlns:ns3='http://blog.frankel.ch/ws-inject'><person><id>"
					+ j
					+ "</id><firstName>John</firstName><lastName>Doe</lastName><birthdate>"
					+ xmlCalendar.toXMLFormat()
					+ "</birthdate></person></ns3:findPersonResponseType>";

			data[i] = new Object[] { request, expectedResponse };
		}

		return data;
	}

	@Test(dataProvider = "data")
	public void findRequestPayloadShouldBeSameAsExpected(String request, String expectedResponse)
			throws DatatypeConfigurationException {

		Source requestPayload = new StringSource(request);

		Source expectedResponsePayload = new StringSource(expectedResponse);

		client.sendRequest(withPayload(requestPayload)).andExpect(payload(expectedResponsePayload));
	}
}
